// Lattice gas.

#ifndef _LATTICE_GAS_HEADER
#define _LATTICE_GAS_HEADER

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>

#define WORDLENGTH 64
typedef uint64_t bitslice;

typedef struct {
	bitslice in[6];
} cell_slice_t;

typedef struct {
	int width;
	int height;
	int state;
	cell_slice_t* buffer0;
	cell_slice_t* buffer1;
} space_t;

space_t allocate_space(int width, int height);
void update(const space_t* restrict s);
void time_reverse(const space_t* restrict s);
void init_lfsr(void);
uint64_t update_random(void);
uint64_t full_random(void);

extern int scattering;

#ifdef COUNT_RANDOM_BITS_FOR_FUN
extern long random_bits_consumed;
#endif

// This little block of logic is here as apposed to in example.c, because it affects both example.c and model.c
#ifdef PSYCHEDELIC
#warning Compiling in psychedelic mode.
#define LOW_PASS
#define BOX_IN_THE_MIDDLE
#define INITIAL_INTERLACE_FACTOR 1
#define INITIAL_PAUSED 0
#define INITIAL_RATE 4
#define RESET_LAYERS 500
#undef OVERLAY_MOMENTUM
#undef WIND_TUNNEL_BOUNDARY_CONDITIONS
#endif

#endif

